# -*- coding: utf-8 -*-
"""
RoboMentor_Client: Python library and framework for RoboMentor_Client.
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
:copyright: (c) 2020 by RoboMentor.
:license: MIT, see LICENSE for more details.
"""

import logging

class Log:

    logging.basicConfig(
        level=logging.DEBUG,
        format="[%(levelname)s][%(asctime)s][%(filename)s(line:%(lineno)d)]: %(message)s"
    )

    @staticmethod
    def debug(content):
        logging.debug(content)

    @staticmethod
    def info(content):
        logging.info(content)

    @staticmethod
    def warning(content):
        logging.warning(content)

    @staticmethod
    def error(content):
        logging.error(content)



# -*- coding: utf-8 -*-
"""
RoboMentor_Client: Python library and framework for RoboMentor_Client.
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
:copyright: (c) 2020 by RoboMentor.
:license: MIT, see LICENSE for more details.
"""

import requests
from ..utils import Log

class Service:

    @classmethod
    def api(cls, url, params, headers, http_method):
        res = ""
        if http_method.upper() == "POST":
            try:
                res = requests.post(url, params, headers=headers)
            except Exception as e:
                Log.error("POST请求出现了异常：{0}".format(e))
        elif http_method.upper() == "GET":
            try:
                res = requests.get(url, params, headers=headers)
            except Exception as e:
                Log.error("GET请求出现了异常：{0}".format(e))

        return res


# -*- coding: utf-8 -*-
"""
RoboMentor_Client: Python library and framework for RoboMentor_Client.
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
:copyright: (c) 2020 by RoboMentor.
:license: MIT, see LICENSE for more details.
"""

from .log import Log



# -*- coding: utf-8 -*-
"""
RoboMentor_Client: Python library and framework for RoboMentor_Client.
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
:copyright: (c) 2020 by RoboMentor.
:license: MIT, see LICENSE for more details.
"""

import serial
from ..utils import Log

class Serial:

    def __init__(self, port, baudrate, size):

        self.conn = serial.Serial()
        self.conn.port = port
        self.conn.baudrate = baudrate
        self.conn.bytesize = size
        self.conn.stopbits = serial.STOPBITS_ONE
        self.conn.parity = serial.PARITY_NONE
        self.conn.timeout = 0.2
        self.conn.open()

    def write(self, command):

        try:
            self.conn.write(command)
        except Exception as e:
            Log.error(e)

    def read(self):

        read_data = self.conn.readall()

        return read_data.decode('utf-8')

    def close(self):

        self.conn.close()




# -*- coding: utf-8 -*-
"""
RoboMentor_Client: Python library and framework for RoboMentor_Client.
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
:copyright: (c) 2020 by RoboMentor.
:license: MIT, see LICENSE for more details.
"""

import sys
import os
import socket
import re
import uuid
import time
import json
import platform
import threading
from .utils import Log
from .service import Message, Service, System
from .function import Serial
from .device import GPIO
from .__config__ import __apiUrl__, __messageUrl__, __version__

if len(sys.argv) < 3:
    Log.error("Input app_id OR app_secret Error")
    sys.exit(0)

class Init:

    def __init__(self):

        Log.info("RoboMentor_Client " + __version__)

        auth_time = time.strftime('%Y%m%d%H%M', time.localtime(time.time()))

        headers = {
            "Content-Type": "application/json",
            "Robot-Token": sys.argv[1] + "@" + sys.argv[2] + "@" + auth_time
        }

        params_ip = self.get_host_ip()

        params_mac = self.get_mac_address()

        params_platform = self.get_platform()

        params = {"app_id": sys.argv[1], "app_secret": sys.argv[2], "robot_mac": params_mac, "robot_ip": params_ip, "robot_platform": params_platform, "robot_version": __version__}

        res = Service.api(__apiUrl__ + "/oauth/robot/register", params, headers, 'GET')

        res_json = res.json()

        assert res_json["code"] == 0, Log.error("Robot Init Error")

        if res_json["code"] != 0:
            sys.exit(0)

        self.app_id = str(sys.argv[1])
        self.app_secret = str(sys.argv[2])
        self.ip = str(params_ip)
        self.mac = str(params_mac)
        self.token = str(res_json["data"]["token"])
        self.name = str(res_json["data"]["robot_title"])
        self.net_ip = str(res_json["data"]["robot_net_ip"])
        self.version = str(__version__)
        self.message = Message(__messageUrl__, self.mac, self.app_id, self.app_secret).start()
        self.system_task_thread = threading.Thread(target=self.system_task)
        self.system_task_thread.start()
        self.robot_task_thread = threading.Thread(target=self.robot_task)
        self.robot_task_thread.start()
        notice_data = {"message_type": "robot_run", "robot_run": {"type": "start_success"}}
        self.message.publish(json.dumps(notice_data))

    def system_task(self):
        system = System()
        while True:
         notice_data = {"message_type": "system_message", "system_message": {"time": time.strftime('%H:%M:%S', time.localtime(time.time())), "cpu": system.get_cpu_info(), "memory": system.get_memory_info()}}
         self.message.publish(json.dumps(notice_data))
         time.sleep(2)

    def robot_task(self):
        while True:
            robot_message = self.message.read_robot_message()
            if robot_message is not None:
                robot_message_json = json.loads(robot_message)
                if robot_message_json["message_type"] == "robot_config":
                    robot_config = json.loads(robot_message_json["robot_config"]["content"])
                    self.name = robot_config["robot_title"]
                if robot_message_json["message_type"] == "robot_run":
                    if robot_message_json["robot_run"]["type"] == "update":
                        os.system("sudo cp /robot/RoboMentor_Client/robot.py /robot/RoboMentor_Client/robot.bak")
                        robot_file = open("/robot/RoboMentor_Client/robot.py", 'w')
                        robot_file.write(robot_message_json["robot_run"]["content"])
                        robot_file.close()
                        notice_data = {"message_type":"robot_run","robot_run":{"type":"update_success"}}
                        self.message.publish(json.dumps(notice_data))
                    if robot_message_json["robot_run"]["type"] == "restart":
                        robot_restart = os.popen("sudo sh /robot/RoboMentor_Client/robot_restart.sh " + self.app_id + " " + self.app_secret).read()
                        if robot_restart != "success":
                            notice_data = {"message_type": "robot_run", "robot_run": {"type": "start_error"}}
                            self.message.publish(json.dumps(notice_data))
                if robot_message_json["message_type"] == "serial_message":
                    serial_conn = Serial(robot_message_json["serial_message"]["port"], int(robot_message_json["serial_message"]["rate"]), int(robot_message_json["serial_message"]["size"]))
                    serial_conn.write(robot_message_json["serial_message"]["content"].encode('utf-8'))
                    if robot_message_json["serial_message"]["switch"] :
                        serial_data = serial_conn.read()
                        notice_data = {"message_type": "serial_message_read", "serial_message_read": {"content": serial_data}}
                        self.message.publish(json.dumps(notice_data))
                if robot_message_json["message_type"] == "pwm_message":
                    serial_conn = Serial(robot_message_json["pwm_message"]["port"], int(robot_message_json["pwm_message"]["rate"]), int(robot_message_json["pwm_message"]["size"]))
                    pwm_data = {"type":robot_message_json["pwm_message"]["type"], "channel":int(robot_message_json["pwm_message"]["channel"]), "width":robot_message_json["pwm_message"]["width"], "rate": int(robot_message_json["pwm_message"]["hz"])}
                    serial_conn.write(json.dumps(pwm_data).encode('utf-8'))

    @staticmethod
    def get_host_ip():
        try:
            s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
            s.connect(('8.8.8.8', 80))
            ip_adds = s.getsockname()[0]
        finally:
            s.close()
        return ip_adds

    @staticmethod
    def get_mac_address():
        return ":".join(re.findall(r".{2}", uuid.uuid1().hex[-12:]))

    @staticmethod
    def get_platform():
        return platform.system() + " " + platform.machine()


# -*- coding: utf-8 -*-
"""
RoboMentor_Client: Python library and framework for RoboMentor_Client.
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
:copyright: (c) 2020 by RoboMentor.
:license: MIT, see LICENSE for more details.
"""

import psutil

class System:

    def __init__(self):
        self.cpu_percent = 0.00
        self.memory_percent = 0.00

    def get_cpu_info(self):
        self.cpu_percent = psutil.cpu_percent(interval=1)
        return self.cpu_percent

    def get_memory_info(self):
        virtual_memory = psutil.virtual_memory()
        self.memory_percent = virtual_memory.percent
        return self.memory_percent




# -*- coding: utf-8 -*-
"""
RoboMentor_Client: Python library and framework for RoboMentor_Client.
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
:copyright: (c) 2020 by RoboMentor.
:license: MIT, see LICENSE for more details.
"""

import json
import paho.mqtt.client as mqtt
from queue import Queue, PriorityQueue


class Message:

    def __init__(self, host, client_id, username, password):
        self.host = host
        self.port = 1883
        self.client_id = client_id
        self.username = username
        self.password = password
        self.mqtt_client = None
        self.timeout = 60
        self.remote_message = Queue(100)
        self.robot_message = PriorityQueue(100)

    def on_connect(self, client, userdata, flags, rc):
        self.subscribe("robot/" + self.client_id)

    def on_message(self, client, userdata, msg):
        message_data = json.loads(msg.payload)
        if message_data["message_type"] == "remote_message":
            self.remote_message.put(str(msg.payload, encoding="utf-8"))
        else:
            self.robot_message.put(str(msg.payload, encoding="utf-8"))

    def read_remote_message(self):
        data = None
        if not self.remote_message.empty():
            data = self.remote_message.get()
        return data

    def read_robot_message(self):
        data = None
        if not self.robot_message.empty():
            data = self.robot_message.get()
        return data

    def subscribe(self, topic):
        self.mqtt_client.subscribe(topic, 0)

    def publish(self, blob):
        self.mqtt_client.publish("robot/" + self.client_id, blob)

    def start(self):
        if self.mqtt_client is None:
            self.mqtt_client = mqtt.Client("robot-" + self.client_id)
            self.mqtt_client.username_pw_set(self.username, self.password)
            self.mqtt_client.on_connect = self.on_connect
            self.mqtt_client.on_message = self.on_message
            self.mqtt_client.connect(self.host, self.port, self.timeout)
            self.mqtt_client.loop_start()

        return self

    def stop(self):
        if self.mqtt_client is not None:
            self.mqtt_client.loop_stop()
            self.mqtt_client.disconnect()
            self.mqtt_client = None
